using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.AnalysisServices;

namespace AmoImpactAnalysis
{
	class Program
	{
		static void Main(string[] args)
		{
			//Create a server object.
			using (Server srv = new Server())
			{
				//Connect to the server.
				srv.Connect("localhost");
				//Chose the database.
				Database db = srv.Databases.GetByName("Foodmart 2005");
                //Add the attribute named Marital Status from the Customer dimension.
                Dimension dbDim = db.Dimensions.GetByName("Customer");
                DimensionAttribute attr = dbDim.Attributes.GetByName("Marital Status");
                dbDim.Attributes.Remove(attr);
                
				//Allocate the collection for the warning that might be returned to the application.
				XmlaWarningCollection warnings = new XmlaWarningCollection();
				//Allocate the collection for the details of impact analysis.
				ImpactDetailCollection impactDetalCollection = new ImpactDetailCollection();
				//Analyze the objects that will be affected if the database were updated, but dont update the database.
				db.Update(UpdateOptions.ExpandFull, UpdateMode.Default, warnings, 
                    impactDetalCollection, true);

				//Iterate the collection of impact details and print: impacted objects, impact, and description of the impact.
				foreach (ImpactDetail impactDetail in impactDetalCollection)
				{
                    Console.WriteLine(impactDetail.Object.Name + "\t" + impactDetail.Object.GetType() + "\t" + impactDetail.Impact.ToString() + "\t" + impactDetail.Description);
				}
			}
		}
	}
}
