using System;
using System.ComponentModel;
using System.Windows.Forms;
using Microsoft.AnalysisServices;

namespace AmoProcessCancel
{
	public partial class Form1 : Form
	{
		Server server = null;
		BackgroundWorker backgroundWorker = null;
		
		public Form1()
		{
			InitializeComponent();
            //Before the connection is established, only the "Connection" button is enabled
            connectionButton.Enabled = true;
            cancelButton.Enabled = false;
            processButton.Enabled = false;
            //Create BackgroundWorker object
            this.backgroundWorker = new BackgroundWorker();
            this.backgroundWorker.WorkerReportsProgress = false;
            this.backgroundWorker.WorkerSupportsCancellation = true;
            this.backgroundWorker.RunWorkerCompleted += OnCompleted;
            this.backgroundWorker.DoWork += OnDoWork; 
		}

		//This function is called when the user clicks the Connection button 
		private void connectionButton_Click(object sender, EventArgs e)
		{
			//Open a connection to the server
			this.server = new Server();
			this.server.Connect("Datasource=localhost;");

			// When the connection is opened, it is possible to start processing, 
            //but still impossible to cancel the process
			connectionButton.Enabled = false;
			processButton.Enabled = true;
		}

		// This function is called when the user clicks the Process button 	
		private void processButton_Click(object sender, EventArgs e)
		{
			// Get a database object and cube object.
			Database database = this.server.Databases.GetByName("Foodmart 2005");
			Cube cube = database.Cubes.GetByName("Warehouse and Sales");
			// Before starting the worker thread, disable the Process button so 
            // the user wont be able to click it again. 
			//Enable the Cancel button
			cancelButton.Enabled = true;
			processButton.Enabled = false;
			// Start the worker thread, then pass the cube object 
            // as a parameter to the function that will be called on the worker thread
			this.backgroundWorker.RunWorkerAsync(cube);
		}

		// This function is called when the user clicks the Cancel button
		private void CancelButton_Click(object sender, EventArgs e)
		{
			// Call the Cancel method to send a cancel request to the server
            try
            {
                this.server.CancelCommand(this.server.SessionID);
            }
            catch (OperationException ex)
            {
                //If you send the Cancel command twice Analysis Services might return an error, 
                //that command is already cancelled. We assume that Cancel command can't through other exceptions.
            }
            //Inform the BackgroundWorker that request has been canceled 
            this.backgroundWorker.CancelAsync();
        }

		// This function is called on the background thread
		void OnDoWork(object sender, DoWorkEventArgs doWorkArgs)
        {
			// Retrieve the Database object from the doWorkArgs parameter
			BackgroundWorker backgroundWorker;
			backgroundWorker = sender as BackgroundWorker;
			Cube cube = (Cube)doWorkArgs.Argument;
			//Start the processing
			cube.Process(ProcessType.ProcessFull);
			// If, during the processing, the user clicks the Cancel button, a flag backgroundWorker.CancellationPending is set to true.
			if(backgroundWorker.CancellationPending)
				doWorkArgs.Cancel = true;
			else
				doWorkArgs.Result = true;
		}

		//This function is called by the class BackgroundWorker, on the main thread when the function running in the background completes
		void OnCompleted(object sender, RunWorkerCompletedEventArgs completedArgs)
		{
			// If there was an error, notify the user.
			if(completedArgs.Error != null)
			{
				MessageBox.Show(completedArgs.Error.Message,completedArgs.Error.Source);
			}
			else if (completedArgs.Cancelled)
			{
				MessageBox.Show(" Processing has been canceled by the user ");
			}
			else
			{
				// The processing completed successfully
				MessageBox.Show(" Processing has been completed ");
			}
			//Execution of the command completed, close the connection and reset all buttons.
			if (null != this.server && this.server.Connected)
			{
				//disconnect from the server
				this.server.Disconnect();
			}		
			this.server = null;
			connectionButton.Enabled = true;
			cancelButton.Enabled = false;
			processButton.Enabled = false;
		}
	}
}