using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using Microsoft.AnalysisServices;
using Microsoft.AnalysisServices.AdomdClient;

namespace AmoSessionShareSample
{
	class Program
	{
		static void Main(string[] args)
		{
			try
			{
				// Create an ADOMD.NET Connection object. 
				Microsoft.AnalysisServices.AdomdClient.AdomdConnection adomdConnection = new Microsoft.AnalysisServices.AdomdClient.AdomdConnection();
				adomdConnection.ConnectionString = "Datasource=localhost; Initial Catalog=Foodmart 2005;";
				adomdConnection.Open();

				//Create an AMO Server object.
				Microsoft.AnalysisServices.Server server = new Microsoft.AnalysisServices.Server();
				//Connect to the server using the same session that the ADOMD.NET connection uses.
				server.Connect("Datasource=localhost", adomdConnection.SessionID);

				Microsoft.AnalysisServices.Database database = server.Databases.FindByName("Foodmart 2005");
                //Create a new cube.
				Microsoft.AnalysisServices.Cube cube = CreateCube(database);

				//Start the transaction.
				server.BeginTransaction();
				//Update the cube. 
				cube.Update(UpdateOptions.ExpandFull);
				//Process the cube.
				cube.Process(ProcessType.ProcessDefault);
				//Check that new cube is available through the ADOMD.NET connection.
				//It would not be available if the session werent shared
				if (null != adomdConnection.Cubes.Find(cube.Name))
					Console.WriteLine("The new cube created by AMO is available through ADOMD.");
				//Roll back the transaction.
				server.RollbackTransaction();
				//Close the AMO connection;this call does not end the session.
				server.Disconnect(false);
				//Close the ADOMD.NET connection; this call does close the session.
				adomdConnection.Close();
			}
			catch (AmoException amoException)
			{
				Console.WriteLine(amoException.Message);
			}
			catch (AdomdException adomdException)
			{
				Console.WriteLine(adomdException.Message);
			}
		}
		
		//create a very simple cube
		static private Microsoft.AnalysisServices.Cube CreateCube(Microsoft.AnalysisServices.Database database)
		{
			//Create a new cube.
			Microsoft.AnalysisServices.Cube cube = database.Cubes.Add(database.Cubes.GetNewID(), database.Cubes.GetNewName());
			//Add the reference to the database dimension to the cube.
			Microsoft.AnalysisServices.Dimension dim = database.Dimensions.GetByName("Customer");
			cube.Dimensions.Add(dim.ID);
			//Create a measure group.
			MeasureGroup mg = cube.MeasureGroups.Add("MeasureGroup1");
			mg.StorageMode = StorageMode.Molap;
			mg.ProcessingMode = ProcessingMode.Regular;

			RegularMeasureGroupDimension regMgDim;
			MeasureGroupAttribute mgAttr;
			//Add a measure group dimension.
			regMgDim = new RegularMeasureGroupDimension(dim.ID);
			mg.Dimensions.Add(regMgDim);
			mgAttr = regMgDim.Attributes.Add(dim.Attributes["Customer"].ID);
			mgAttr.Type = MeasureGroupAttributeType.Granularity;
			mgAttr.KeyColumns.Add(CreateDataItem(database.DataSourceViews[0], "dbo_sales_fact_1997", "customer_id"));

			//Create one measure.
			Microsoft.AnalysisServices.Measure meas;
			meas = mg.Measures.Add("Unit Sales");
			meas.AggregateFunction = AggregationFunction.Count;
			meas.FormatString = "#";
			meas.Visible = true;
			meas.Source = CreateDataItem(database.DataSourceViews[0], "dbo_sales_fact_1997", "Unit_Sales");
			return cube;
		}
		static DataItem CreateDataItem(DataSourceView dsv, string tableName, string columnName)
		{
			DataTable dataTable = ((DataSourceView)dsv).Schema.Tables[tableName];
			DataColumn dataColumn = dataTable.Columns[columnName];
			return new DataItem(tableName, columnName, OleDbTypeConverter.GetRestrictedOleDbType(dataColumn.DataType));
		}
	}
}
